﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Drawing;
using System.IO;
using System.Text;
using System.Windows.Forms;

namespace iPodm3uCreator.BL
{
    public class MusicTreeController
    {
        #region constructor
        public MusicTreeController(TreeView treeView)
        {
            _treeView = treeView;
            _treeView.TreeViewNodeSorter = new TreeViewSorter();
            _musicFolder = String.Empty;
            _rootNode = null;
            _extensions = new List<string>();
            if (File.Exists("MusicFileFilters.txt"))
            {
                try
                {
                    _extensions.AddRange(File.ReadAllLines("MusicFileFilters.txt"));
                }
                catch (Exception ex)
                {
                    MessageBox.Show(ex.Message);
                }
                 
            }
            else
            {
                _extensions.Add("*.mp3");
                _extensions.Add("*.wma");
            }  
        }
        #endregion

        #region Members

        private readonly TreeView _treeView;
        private string _musicFolder;
        private List<string> _extensions;
        private TreeNode _rootNode;

        #endregion

        #region Properties

        public string MusicFolder
        {
            get { return _musicFolder; }
        }

        public TreeNode RootNode
        {
            get { return _rootNode; }
        }

        #endregion

        #region Methods
        public void LoadMusicFolder(string musicFolder)
        {

            if (Directory.Exists(musicFolder))
            {
                _musicFolder = musicFolder;
                _treeView.Nodes.Clear();

                string dirName = GetName(musicFolder);
                _rootNode = _treeView.Nodes.Add(dirName, dirName, 0);

                string[] directories = Directory.GetDirectories(musicFolder);
                foreach (string directory in directories)
                {
                    DrawDir(_rootNode, directory);
                }

                List<string> files = GetFiles(musicFolder, _extensions);
                foreach (string file in files)
                {
                    string fileName = GetName(file);
                    _rootNode.Nodes.Add(fileName, fileName, 2);
                }

                
                // structure.txt
                string structureFileName = musicFolder + "\\structure.txt";
                if (File.Exists(structureFileName))
                {
                    List<string> structure = new List<string>();
                    structure.AddRange(File.ReadAllLines(structureFileName));
                    char[] separator = { '/' };

                    foreach (string existedFile in structure)
                    {
                        string[] existedFilePath = existedFile.Split(separator);
                        string tempDir = musicFolder;
                        TreeNode node = _rootNode;

                        for (int i=0; i < existedFilePath.Length; i++)
                        {
                            //dir
                            if (i < existedFilePath.Length-1)
                            {
                                bool isExistedDir = true;
                                if (!node.Nodes.ContainsKey(existedFilePath[i]))
                                {
                                    node.Nodes.Add(existedFilePath[i], existedFilePath[i], 0);
                                    isExistedDir = false;
                                }
                                node = node.Nodes[existedFilePath[i]];
                                if (!isExistedDir)
                                {
                                    node.ForeColor = Color.Blue;
                                }
                            }
                            //file
                            else
                            {
                                bool isExistedFile = true;
                                if (!node.Nodes.ContainsKey(existedFilePath[i]))
                                {
                                    node.Nodes.Add(existedFilePath[i], existedFilePath[i], 2);
                                    isExistedFile = false;
                                }
                                node = node.Nodes[existedFilePath[i]];
                                if (!isExistedFile)
                                {
                                    node.ForeColor = Color.Blue;
                                }
                            }
                        }
                    }    
                }

                _treeView.Sort();
                _rootNode.Checked = true;
                _rootNode.Expand();
            }
        }

        public List<string> GetCheckedFilesWithPath()
        {
            return GetCheckedFilesWithPath(_rootNode, 1);
        }


        public List<string> GetCheckedFilesWithPath(TreeNode startNode, uint startFromLevel)
        {
            if (startNode != null)
            {
                List<string> files = new List<string>();

                foreach (TreeNode node in startNode.Nodes)
                {
                    if ((node.Nodes.Count == 0) && (node.ImageIndex == 2 /*music file*/) && (node.Checked))
                    {
                        files.Add(GetFullPath(node, startFromLevel));
                    }
                    else //folder
                    {
                        files.AddRange(GetCheckedFilesWithPath(node, startFromLevel));
                    }
                }

                return files;
            }
            else
            {
                return new List<string>();
            }
        }

        private void DrawDir(TreeNode parentNode, string dir)
        {
            if (Directory.Exists(dir))
            {
                string dirName = GetName(dir);
                TreeNode dirNode = parentNode.Nodes.Add(dirName, dirName, 0);

                string[] directories = Directory.GetDirectories(dir);
                foreach (string directory in directories)
                {
                    DrawDir(dirNode, directory);
                }

                List<string> files = GetFiles(dir, _extensions);
                foreach (string file in files)
                {
                    string fileName = GetName(file);
                    dirNode.Nodes.Add(fileName, fileName, 2);
                }
            }
        }

        public string GetName(string dir)
        {
            char[] separator = { '\\' };
            string[] dirPaths = dir.Split(separator);
            return dirPaths[dirPaths.Length - 1];
        }

        
        private static string GetFullPath(TreeNode node)
        {
            return GetFullPath(node, 1);
        }
        
        private static string GetFullPath(TreeNode node, uint startFromLevel)
        {
            
            if (node == null)
            {
                return null;
            }

            string fullPath = String.Empty;
            List<string> pathParts = new List<string>();
            while ((node.Parent != null) && (node.Level >= startFromLevel))
            {
                pathParts.Add(node.Text);
                node = node.Parent;
            }

            for (int i=pathParts.Count-1; i>=0; i--)
            {
                fullPath += pathParts[i] + "/";
            }
            fullPath = fullPath.Substring(0, fullPath.Length - 1);
            return fullPath;
        }

        public string getFullWindowsDirPath(TreeNode node)
        {
            return _musicFolder + "\\" + GetFullPath(node).Replace('/', '\\');
        }

        private static List<string> GetFiles(string path, List<string> listOfSearchPatterns)
        {
            List<string> matchingFiles = new List<string>();
            listOfSearchPatterns.ForEach(
                                delegate(string s)
                                {
                                    matchingFiles.AddRange(Directory.GetFiles(path, s));
                                });

            return matchingFiles;

        }
        #endregion

        #region TreeViewSorter
        private class TreeViewSorter : IComparer
        {
            public int Compare(object x, object y)
            {
                TreeNode xNode = (TreeNode) x;
                TreeNode yNode = (TreeNode) y;
                if ((xNode.ImageIndex != 2) && (yNode.ImageIndex == 2))
                {
                    return -1;
                }
                if ((xNode.ImageIndex == 2) && (yNode.ImageIndex != 2))
                {
                    return 1;
                }

                List<string> strings = new List<string>();
                strings.Add(xNode.Text);
                strings.Add(yNode.Text);
                strings.Sort();

                if (xNode.Text == strings[0])
                {
                    return -1;
                }
                return 1;
           
            }
        }

        #endregion
    }
}
